// Project  STOPWATCH x 10
// Compiler SDCC 4.1.0 #12072 (MINGW64)
// Low-level "core" code based on work work by Stephen Erisman <github@serisman.com>
// 20230703 M. Ray Burnette using Windows 11 64-bit Professional

#include "Arduino.h"					// Implement the construct of setup() and loop()
#include "hal.h"						// #include "cc253x/cc253x.h"
#include "util.h"						// Compile macros
#include "string_utils.h"				// prototypes for all *_str.c functions
#include "clock.h"						// Setup for millis and micros via Interrupt
#include "display.h"					// buffer screen[NUM_SCREEN_ROWS * SCREEN_WIDTH]
#include "keypad.h"						// keymap array to button-by-name defines, keypad_scan, keypad_poll, etc

void nextFrame(void);
void displayShow();
void TimeFormat(uint32_t, int);

#define FPS	30u							// LCD screen buffer update

/*
SDCC data types: char (8 bits, 1 byte), short (16 bits, 2 bytes), 
				int (16 bits, 2 bytes), long (32 bit, 4 bytes), 
				long long (64 bit, 8 bytes), float (4 byte IEEE),
				and _Bool/bool.
*/
// __data indication to SDCC to use "fast SRAM" where __xdata is generic SRAM

char		__xdata str_buf[12];
bool		__xdata flag0, flag1, flag2, flag3, flag4, flag5, flag6, flag7, flag8, flag9;
bool		__xdata PauseFlag;
uint32_t 	__xdata T0, T1, T2, T3, T4, T5, T6, T7, T8, T9;

uint32_t 	__xdata millis, minutes, seconds;

void setup() {
	oscillator_32mhz();					// defined in oscillator.h
	clock_init();
	ENABLE_INTERRUPTS
	display_init();
	display_set_frame_rate(FPS);

	flag0= false; flag1= false; flag2= false; flag3= false; flag4= false; 
	flag5= false; flag6= false; flag7= false; flag8= false; flag9= false;
	PauseFlag = false;
}

void loop() {
	if (display_next_frame())			// Wait until its time to render our next frame
	{
		nextFrame();
	}
}

void nextFrame() {
	keypad_poll();						// Poll the keypad
	if (keypad_any_pressed())
	{
		for (uint8_t col=0; col<4; col++) {
		  for (uint8_t row=0; row<8; row++) {
			uint8_t mask = util_bit_to_mask[row];
			if (keypad_pressed(col, mask)) {
			  uint32_t Results = (uint32_t)(row * 5 + col);

			  switch (Results)
				{

				case 0:					// Enter
					PauseFlag = !(PauseFlag);
					clock_delay_ms(150);
					break;
/*
				case 1:					// Lf arrow 					

					break;
				case 2:					// Rt arrow

					break;
				case 3:					// del/Backspace

					break;
			*/		
				case 5:					// ?
					display_init();
					clock_delay_ms(1000);
					display_set_cursor(0, 20);
					display_print("20230703");
					display_set_cursor(0,30);
					display_print("by Ray Burne");
					display_paint(true);
					clock_delay_ms(150000);
					break;

				case 6:					// A/1
					if(flag1 == false)	T1 = clock_millis();
					flag1 = true;
					break;
				case 7:					// B/2
					if(flag2 == false)	T2 = clock_millis();
					flag2 = true;
					break;
				case 8:					// C/3
					if(flag3 == false)	T3 = clock_millis();
					flag3 = true;
					break;

				case 10:				// Home == clear all counters
					T0 = 0;	T1 = 0;	T2 = 0;	T3 = 0;	T4 = 0; 
					T5 = 0;	T6 = 0;	T7 = 0;	T8 = 0;	T9 = 0;
					flag0= false; flag1= false; flag2= false; flag3= false; flag4= false; 
					flag5= false; flag6= false; flag7= false; flag8= false; flag9= false;
					PauseFlag = false;
					display_init();
					break;
				case 11:				// D/4
					if(flag4 == false)	T4 = clock_millis();
					flag4 = true;
					break;
				case 12:				// E/5
					if (flag5 == false)	T5 = clock_millis();
					flag5 = true;
					break;
				case 13:				// F/6
					if (flag6 == false)	T6 = clock_millis();
					flag6 = true;
					break;
				case 15:				// Up

					break;
				case 16:				// G/7
					if (flag7 == false)	T7 = clock_millis();
					flag7 = true;
					break;
				case 17:				// H/8
					if (flag8 == false)	T8 = clock_millis();
					flag8 = true;
					break;
				case 18:				// I/9
					if(flag9 == false)	T9 = clock_millis();
					flag9 = true;
					break;
/*
				case 20:				// Down

					break;
				case 21:				// +/-

					break;
*/
				case 22:				// J/0
					if(flag0 == 0)		T0 = clock_millis();
					flag0 += 1;
					break;
/*
				case 23:				// x-y/.

					break;
				case 35:				// Power

					break;
*/
				default:
					break;
				}
			}
		  }
		}
		 clock_delay_ms(10);			// debounce: elastomer keyboard sucks
	}
	displayShow();
}


void displayShow() 
{
	if (flag1) {
		display_set_cursor(0,0);	TimeFormat(T1, 181);	}	// line 1 left
	if (flag2) {
		display_set_cursor(70,0);	TimeFormat(T2, 182);	}	// line 1 right
	if (flag3) {
		display_set_cursor(0,10);	TimeFormat(T3, 184);	}	// line 2 leftt
	if (flag4) {
		display_set_cursor(70,10);	TimeFormat(T4, 185);	}	// line 2 right
	if (flag5) {
		display_set_cursor(0,20);	TimeFormat(T5, 187);	}	// line 3 left
	if (flag6) {
		display_set_cursor(70,20);	TimeFormat(T6, 189);	}	// line 3 right
	if (flag7) {
		display_set_cursor(0,30);	TimeFormat(T7, 191);	}	// line 4 left
	if (flag8) {
		display_set_cursor(70,30);	TimeFormat(T8, 193);	}	// line 4 right
	if (flag9) {
		display_set_cursor(0,40);	TimeFormat(T9, 195);	}	// line 4 left
	if (flag0) {
		display_set_cursor(70,40);	TimeFormat(T0, 196);	}	// line 5 right
		
	if(! PauseFlag) {											// Use screen memory to pause updates
		display_paint(true);
	}
}

void TimeFormat(uint32_t time, int OffSetmS) {		// off-set in mS based on empirical data

	time = clock_millis() - time;				// running event time Tx

	seconds = time / 1000;							// running seconds SS
	minutes = seconds / 60;							// running minutes MM
	millis	= time;									// integer milliseconds xxx
	
	seconds %= 60;									// integer seconds
	minutes %= 60;									// integer minutes
	millis  %= 1000;								// integer 3 digits mS

	if (millis > OffSetmS) {
		millis  -= OffSetmS;						// attempt to account for uC processing delays
	} else if (seconds > 0) {
		seconds -= 1;								// borrow 1000 mS
		millis = millis + 1000 - OffSetmS;			// nasty correction: should muck with minutes too
	} else {
		millis = 0;
		if (minutes < 10) {
			display_print("0");
		}
	}

	u32_to_str(str_buf, minutes);
	display_print(str_buf);
	display_print(":");

	if (seconds < 10) {
		display_print("0");
	}

	u32_to_str(str_buf, seconds);
	display_print(str_buf);
	display_print(".");
	u32_to_str(str_buf, millis); display_print(str_buf);
}
